##############################################################################
#
# Copyright (c) 2001, 2002 Zope Corporation and Contributors.
# All Rights Reserved.
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
# 
##############################################################################
"""Global Translation Service for providing I18n to file-based code.

$Id: GlobalTranslationService.py,v 1.3 2002/08/24 15:46:07 lalo Exp $
"""

from Negotiator import negotiator
from SimpleTranslationService import SimpleTranslationService

# The configure.zcml file should specify a list of fallback languages for the
# site.  If a particular catalog for a negotiated language is not available,
# then the zcml specified order should be tried.  If that fails, then as a
# last resort the languages in the following list are tried.  If these fail
# too, then the msgid is returned.
#
# Note that these fallbacks are used only to find a catalog.  If a particular
# message in a catalog is not translated, tough luck, you get the msgid.
LANGUAGE_FALLBACKS = ['en']


class GlobalTranslationService(SimpleTranslationService):

    def __init__(self, default_domain='global', fallbacks=None):
        # XXX We haven't specified that ITranslationServices have a default
        # domain.  So far, we've required the domain argument to .translate()
        self._domain = default_domain
        # _catalogs maps (language, domain) to IMessageCatalog instances
        self._catalogs = {}
        # _data maps IMessageCatalog.getIdentifier() to IMessageCatalog
        self._data = {}
        # What languages to fallback to, if there is no catalog for the
        # requested language (no fallback on individual messages)
        if fallbacks is None:
            fallbacks = LANGUAGE_FALLBACKS
        self._fallbacks = fallbacks

    def _registerMessageCatalog(self, language, domain, catalog_name):
        http_language = language.lower().replace('_', '-')
        key = (http_language, domain)
        mc = self._catalogs.setdefault(key, [])
        mc.append(catalog_name)
        print 'adding catalog for domain %s, language %s' % (domain, language)

    def addCatalog(self, catalog):
        self._data[catalog.getIdentifier()] = catalog
        self._registerMessageCatalog(catalog.getLanguage(),
                                     catalog.getDomain(),
                                     catalog.getIdentifier())

    def setLanguageFallbacks(self, fallbacks=None):
        if fallbacks is None:
            fallbacks = LANGUAGE_FALLBACKS
        self._fallbacks = fallbacks


    def translate(self, domain, msgid, mapping=None, context=None,  
                  target_language=None):
        """
        """
        print 'translating %s:"%s" to %s' % (domain, msgid, target_language)
        if target_language is None:
            if context is None:
                raise TypeError, 'No destination language'
            else:
                langs = [m[0] for m in self._catalogs.keys()]
                target_language = negotiator.getLanguage(langs, context)
                print 'ooops, I mean, to', target_language

        # Get the translation. Use the specified fallbacks if this fails
        catalog_names = self._catalogs.get((target_language, domain))
        if catalog_names is None:
            for language in self._fallbacks:
                catalog_names = self._catalogs.get((language, domain))
                if catalog_names is not None:
                    break
        # Did the fallback fail?  Sigh, use the msgid
        if catalog_names is None:
            catalog_names = []

        text = msgid
        for name in catalog_names:
            catalog = self._data[name]
            text = catalog.queryMessage(msgid)

        # Now we need to do the interpolation
        text = self.interpolate(text, mapping)
        print 'result:', repr(text)
        return text

    #
    ############################################################


translationService = GlobalTranslationService()
