# handle the i18n: namespace

i18n_ns = 'http://xml.zope.org/namespaces/tal/i18n'

from pax.paxtransform import AttributeHandler, StopTransform
from pax import XML
from pax.backwards_compatibility import *
import os, glob
from GlobalTranslationService import GlobalTranslationService
from GettextMessageCatalog import GettextMessageCatalog
import zLOG, sys

# special marker (we're using 0 because it's pickle-friendly)
_no_domain = 0

# default (gettext) translation service
translation_service = GlobalTranslationService('default')
basepath = os.path.join(INSTANCE_HOME, 'i18n')
if os.access(basepath, os.F_OK):
    for name in glob.glob(os.path.join(basepath, '*.mo')):
        try:
            translation_service.addCatalog(GettextMessageCatalog(name))
        except ValueError:
            zLOG.LOG('AltPTi18n', zLOG.PROBLEM, 'Message Catalog has no metadata',
                     name, sys.exc_info())


class Literal(object):
    def __init__(self, text):
        self.text = text

    def __xml__(self, *a):
        return self.text


class I18nHandler(object):
    def __init__(self):
        self.name = 'i18n'
        self.ns = i18n_ns

    def initialize(self, context):
        context.i18n_info_stack = []
        context.i18n_domain_stack = []
        # here we can hook to placeful translation services if we later wish to
        context.translate = translation_service.translate

    def __call__(self, element, context):
        context.i18n_info_stack.append(element.attributes[i18n_ns])
        context.i18n_domain_stack.append(element.i18n_domain)
        return element

    def postprocess(self, element, context):
        # this one does the real work
        # this is done on postprocess and not in a transform to better interact
        # with tal:replace, tal:omit-tag, etc
        if not element:
            # oops - we have been removed, probably by tal:condition: bail out
            return element
        info = context.i18n_info_stack.pop()
        domain = context.i18n_domain_stack.pop()
        #FIXME: domain is TALES?
        if info.has_key('translate') or info.has_key('attributes'):
            if info.has_key('target'):
                target = context.engine.evaluate(info['target'])
            else:
                target = None
        if info.has_key('translate'):
            msgid = info['translate']
            if msgid == '':
                if hasattr(element, 'children'):
                    msgid = XML(element.children)
                else:
                    try:
                        msgid = XML(element)
                    except:
                        msgid = str(element)
                msgid = ' '.join(msgid.split())
            if info.has_key('data'):
                data = context.engine.evaluate(info['data'])
            else:
                data = {}
                # FIXME: populate from sub-elements
            result = Literal(context.translate(domain, msgid, data,
                                               context, target))
            if hasattr(element, 'children'):
                element.children = [result]
            else:
                element = result
        if info.has_key('attributes') and hasattr(element, 'attributes'):
            attrs = element.attributes.setdefault(element.ns, {})
            # FIXME: the spec doesn't say what is the separator!!!
            for name in info['attributes'].split():
                attrs[name] = context.translate(domain, attrs[name], None,
                                                context, target)
        return element

i18n_handler = I18nHandler()


class I18nCompiler(object):
    '''The domain attribute is stored in each element; this results in lexical
    domain scope, which results in better i18n/metal interaction'''
    def __init__(self):
        self.name = 'i18n'
        self.ns = i18n_ns

    def initialize(self, context):
        context.i18n_domain_stack = [None]

    def __call__(self, element, context):
        domain = element.attributes[i18n_ns].get('domain', _no_domain)
        context.i18n_domain_stack.append(domain)
        if domain is _no_domain:
            domain = [d for d in context.i18n_domain_stack if d is not _no_domain][-1]
        element.i18n_domain = domain
        return element

    def postprocess(self, element, context):
        context.i18n_domain_stack.pop()
        return element

i18n_compiler = I18nCompiler()
